# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grid/v1/logs.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name='grid/v1/logs.proto',
    package='grid.v1',
    syntax='proto3',
    serialized_options=b'Z0github.com/gridai/grid/grid-backend/apis/grid/v1',
    create_key=_descriptor._internal_create_key,
    serialized_pb=
    b'\n\x12grid/v1/logs.proto\x12\x07grid.v1\x1a\x1fgoogle/protobuf/timestamp.proto\"\xd0\x01\n\x08LogEntry\x12\x18\n\x07message\x18\x01 \x01(\tR\x07message\x12\x38\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\ttimestamp\x12\x35\n\x06labels\x18\x03 \x03(\x0b\x32\x1d.grid.v1.LogEntry.LabelsEntryR\x06labels\x1a\x39\n\x0bLabelsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\xa6\x02\n\x0fLogQueryRequest\x12\x43\n\x0fstart_timestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0estartTimestamp\x12?\n\rend_timestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0c\x65ndTimestamp\x12\x14\n\x05limit\x18\x03 \x01(\x04R\x05limit\x12<\n\x06labels\x18\x04 \x03(\x0b\x32$.grid.v1.LogQueryRequest.LabelsEntryR\x06labels\x1a\x39\n\x0bLabelsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x32Z0github.com/gridai/grid/grid-backend/apis/grid/v1b\x06proto3',
    dependencies=[
        google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,
    ]
)

_LOGENTRY_LABELSENTRY = _descriptor.Descriptor(
    name='LabelsEntry',
    full_name='grid.v1.LogEntry.LabelsEntry',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='key',
            full_name='grid.v1.LogEntry.LabelsEntry.key',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='key',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='value',
            full_name='grid.v1.LogEntry.LabelsEntry.value',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='value',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=b'8\001',
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=216,
    serialized_end=273,
)

_LOGENTRY = _descriptor.Descriptor(
    name='LogEntry',
    full_name='grid.v1.LogEntry',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='message',
            full_name='grid.v1.LogEntry.message',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='message',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='timestamp',
            full_name='grid.v1.LogEntry.timestamp',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='timestamp',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='labels',
            full_name='grid.v1.LogEntry.labels',
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='labels',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[
        _LOGENTRY_LABELSENTRY,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=65,
    serialized_end=273,
)

_LOGQUERYREQUEST_LABELSENTRY = _descriptor.Descriptor(
    name='LabelsEntry',
    full_name='grid.v1.LogQueryRequest.LabelsEntry',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='key',
            full_name='grid.v1.LogQueryRequest.LabelsEntry.key',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='key',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='value',
            full_name='grid.v1.LogQueryRequest.LabelsEntry.value',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='value',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=b'8\001',
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=216,
    serialized_end=273,
)

_LOGQUERYREQUEST = _descriptor.Descriptor(
    name='LogQueryRequest',
    full_name='grid.v1.LogQueryRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='start_timestamp',
            full_name='grid.v1.LogQueryRequest.start_timestamp',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='startTimestamp',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='end_timestamp',
            full_name='grid.v1.LogQueryRequest.end_timestamp',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='endTimestamp',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='limit',
            full_name='grid.v1.LogQueryRequest.limit',
            index=2,
            number=3,
            type=4,
            cpp_type=4,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='limit',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='labels',
            full_name='grid.v1.LogQueryRequest.labels',
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='labels',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[
        _LOGQUERYREQUEST_LABELSENTRY,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=276,
    serialized_end=570,
)

_LOGENTRY_LABELSENTRY.containing_type = _LOGENTRY
_LOGENTRY.fields_by_name['timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_LOGENTRY.fields_by_name['labels'].message_type = _LOGENTRY_LABELSENTRY
_LOGQUERYREQUEST_LABELSENTRY.containing_type = _LOGQUERYREQUEST
_LOGQUERYREQUEST.fields_by_name['start_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_LOGQUERYREQUEST.fields_by_name['end_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_LOGQUERYREQUEST.fields_by_name['labels'].message_type = _LOGQUERYREQUEST_LABELSENTRY
DESCRIPTOR.message_types_by_name['LogEntry'] = _LOGENTRY
DESCRIPTOR.message_types_by_name['LogQueryRequest'] = _LOGQUERYREQUEST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

LogEntry = _reflection.GeneratedProtocolMessageType(
    'LogEntry',
    (_message.Message, ),
    {
        'LabelsEntry':
        _reflection.GeneratedProtocolMessageType(
            'LabelsEntry',
            (_message.Message, ),
            {
                'DESCRIPTOR': _LOGENTRY_LABELSENTRY,
                '__module__': 'grid.v1.logs_pb2'
                # @@protoc_insertion_point(class_scope:grid.v1.LogEntry.LabelsEntry)
            }
        ),
        'DESCRIPTOR':
        _LOGENTRY,
        '__module__':
        'grid.v1.logs_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.LogEntry)
    }
)
_sym_db.RegisterMessage(LogEntry)
_sym_db.RegisterMessage(LogEntry.LabelsEntry)

LogQueryRequest = _reflection.GeneratedProtocolMessageType(
    'LogQueryRequest',
    (_message.Message, ),
    {
        'LabelsEntry':
        _reflection.GeneratedProtocolMessageType(
            'LabelsEntry',
            (_message.Message, ),
            {
                'DESCRIPTOR': _LOGQUERYREQUEST_LABELSENTRY,
                '__module__': 'grid.v1.logs_pb2'
                # @@protoc_insertion_point(class_scope:grid.v1.LogQueryRequest.LabelsEntry)
            }
        ),
        'DESCRIPTOR':
        _LOGQUERYREQUEST,
        '__module__':
        'grid.v1.logs_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.LogQueryRequest)
    }
)
_sym_db.RegisterMessage(LogQueryRequest)
_sym_db.RegisterMessage(LogQueryRequest.LabelsEntry)

DESCRIPTOR._options = None
_LOGENTRY_LABELSENTRY._options = None
_LOGQUERYREQUEST_LABELSENTRY._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grid/v1/user_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2

from grid.protos.grid.v1 import metadata_pb2 as grid_dot_v1_dot_metadata__pb2
from grid.protos.grid.v1 import user_pb2 as grid_dot_v1_dot_user__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name='grid/v1/user_service.proto',
    package='grid.v1',
    syntax='proto3',
    serialized_options=b'Z0github.com/gridai/grid/grid-backend/apis/grid/v1',
    create_key=_descriptor._internal_create_key,
    serialized_pb=
    b'\n\x1agrid/v1/user_service.proto\x12\x07grid.v1\x1a\x1cgoogle/api/annotations.proto\x1a\x16grid/v1/metadata.proto\x1a\x12grid/v1/user.proto\"6\n\x11\x43reateUserRequest\x12!\n\x04user\x18\x01 \x01(\x0b\x32\r.grid.v1.UserR\x04user\"7\n\x12\x43reateUserResponse\x12!\n\x04user\x18\x01 \x01(\x0b\x32\r.grid.v1.UserR\x04user\" \n\x0eGetUserRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"4\n\x0fGetUserResponse\x12!\n\x04user\x18\x01 \x01(\x0b\x32\r.grid.v1.UserR\x04user\"z\n\x10ListUsersRequest\x12%\n\x0e\x63ontinue_token\x18\x01 \x01(\tR\rcontinueToken\x12)\n\x10resource_version\x18\x02 \x01(\tR\x0fresourceVersion\x12\x14\n\x05limit\x18\x03 \x01(\x03R\x05limit\"\x88\x01\n\x11ListUsersResponse\x12!\n\x04user\x18\x01 \x03(\x0b\x32\r.grid.v1.UserR\x04user\x12%\n\x0e\x63ontinue_token\x18\x02 \x01(\tR\rcontinueToken\x12)\n\x10resource_version\x18\x03 \x01(\tR\x0fresourceVersion\"6\n\x11UpdateUserRequest\x12!\n\x04user\x18\x01 \x01(\x0b\x32\r.grid.v1.UserR\x04user\"7\n\x12UpdateUserResponse\x12!\n\x04user\x18\x01 \x01(\x0b\x32\r.grid.v1.UserR\x04user\"<\n\x17UpdateUserStatusRequest\x12!\n\x04user\x18\x01 \x01(\x0b\x32\r.grid.v1.UserR\x04user\"=\n\x18UpdateUserStatusResponse\x12!\n\x04user\x18\x01 \x01(\x0b\x32\r.grid.v1.UserR\x04user\"6\n\x11\x44\x65leteUserRequest\x12!\n\x04user\x18\x01 \x01(\x0b\x32\r.grid.v1.UserR\x04user\"7\n\x12\x44\x65leteUserResponse\x12!\n\x04user\x18\x01 \x01(\x0b\x32\r.grid.v1.UserR\x04user\"T\n\x11WatchUsersRequest\x12)\n\x10resource_version\x18\x01 \x01(\tR\x0fresourceVersion\x12\x14\n\x05limit\x18\x02 \x01(\x03R\x05limit\"\x8f\x01\n\x12WatchUsersResponse\x12)\n\x10resource_version\x18\x01 \x01(\tR\x0fresourceVersion\x12+\n\x04type\x18\x02 \x01(\x0e\x32\x17.grid.v1.WatchEventTypeR\x04type\x12!\n\x04user\x18\x03 \x01(\x0b\x32\r.grid.v1.UserR\x04user\"3\n\x18StopUserWorkloadsRequest\x12\x17\n\x07user_id\x18\x01 \x01(\tR\x06userId\"\x1b\n\x19StopUserWorkloadsResponse2\xec\x04\n\x0bUserService\x12G\n\nCreateUser\x12\x1a.grid.v1.CreateUserRequest\x1a\x1b.grid.v1.CreateUserResponse\"\x00\x12>\n\x07GetUser\x12\x17.grid.v1.GetUserRequest\x1a\x18.grid.v1.GetUserResponse\"\x00\x12\x44\n\tListUsers\x12\x19.grid.v1.ListUsersRequest\x1a\x1a.grid.v1.ListUsersResponse\"\x00\x12G\n\nUpdateUser\x12\x1a.grid.v1.UpdateUserRequest\x1a\x1b.grid.v1.UpdateUserResponse\"\x00\x12W\n\x10UpdateUserStatus\x12 .grid.v1.UpdateUserStatusRequest\x1a!.grid.v1.UpdateUserStatusResponse\x12G\n\nDeleteUser\x12\x1a.grid.v1.DeleteUserRequest\x1a\x1b.grid.v1.DeleteUserResponse\"\x00\x12G\n\nWatchUsers\x12\x1a.grid.v1.WatchUsersRequest\x1a\x1b.grid.v1.WatchUsersResponse0\x01\x12Z\n\x11StopUserWorkloads\x12!.grid.v1.StopUserWorkloadsRequest\x1a\".grid.v1.StopUserWorkloadsResponseB2Z0github.com/gridai/grid/grid-backend/apis/grid/v1b\x06proto3',
    dependencies=[
        google_dot_api_dot_annotations__pb2.DESCRIPTOR,
        grid_dot_v1_dot_metadata__pb2.DESCRIPTOR,
        grid_dot_v1_dot_user__pb2.DESCRIPTOR,
    ]
)

_CREATEUSERREQUEST = _descriptor.Descriptor(
    name='CreateUserRequest',
    full_name='grid.v1.CreateUserRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='user',
            full_name='grid.v1.CreateUserRequest.user',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='user',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=113,
    serialized_end=167,
)

_CREATEUSERRESPONSE = _descriptor.Descriptor(
    name='CreateUserResponse',
    full_name='grid.v1.CreateUserResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='user',
            full_name='grid.v1.CreateUserResponse.user',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='user',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=169,
    serialized_end=224,
)

_GETUSERREQUEST = _descriptor.Descriptor(
    name='GetUserRequest',
    full_name='grid.v1.GetUserRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='id',
            full_name='grid.v1.GetUserRequest.id',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='id',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=226,
    serialized_end=258,
)

_GETUSERRESPONSE = _descriptor.Descriptor(
    name='GetUserResponse',
    full_name='grid.v1.GetUserResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='user',
            full_name='grid.v1.GetUserResponse.user',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='user',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=260,
    serialized_end=312,
)

_LISTUSERSREQUEST = _descriptor.Descriptor(
    name='ListUsersRequest',
    full_name='grid.v1.ListUsersRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='continue_token',
            full_name='grid.v1.ListUsersRequest.continue_token',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='continueToken',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='resource_version',
            full_name='grid.v1.ListUsersRequest.resource_version',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='resourceVersion',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='limit',
            full_name='grid.v1.ListUsersRequest.limit',
            index=2,
            number=3,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='limit',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=314,
    serialized_end=436,
)

_LISTUSERSRESPONSE = _descriptor.Descriptor(
    name='ListUsersResponse',
    full_name='grid.v1.ListUsersResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='user',
            full_name='grid.v1.ListUsersResponse.user',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='user',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='continue_token',
            full_name='grid.v1.ListUsersResponse.continue_token',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='continueToken',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='resource_version',
            full_name='grid.v1.ListUsersResponse.resource_version',
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='resourceVersion',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=439,
    serialized_end=575,
)

_UPDATEUSERREQUEST = _descriptor.Descriptor(
    name='UpdateUserRequest',
    full_name='grid.v1.UpdateUserRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='user',
            full_name='grid.v1.UpdateUserRequest.user',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='user',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=577,
    serialized_end=631,
)

_UPDATEUSERRESPONSE = _descriptor.Descriptor(
    name='UpdateUserResponse',
    full_name='grid.v1.UpdateUserResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='user',
            full_name='grid.v1.UpdateUserResponse.user',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='user',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=633,
    serialized_end=688,
)

_UPDATEUSERSTATUSREQUEST = _descriptor.Descriptor(
    name='UpdateUserStatusRequest',
    full_name='grid.v1.UpdateUserStatusRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='user',
            full_name='grid.v1.UpdateUserStatusRequest.user',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='user',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=690,
    serialized_end=750,
)

_UPDATEUSERSTATUSRESPONSE = _descriptor.Descriptor(
    name='UpdateUserStatusResponse',
    full_name='grid.v1.UpdateUserStatusResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='user',
            full_name='grid.v1.UpdateUserStatusResponse.user',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='user',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=752,
    serialized_end=813,
)

_DELETEUSERREQUEST = _descriptor.Descriptor(
    name='DeleteUserRequest',
    full_name='grid.v1.DeleteUserRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='user',
            full_name='grid.v1.DeleteUserRequest.user',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='user',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=815,
    serialized_end=869,
)

_DELETEUSERRESPONSE = _descriptor.Descriptor(
    name='DeleteUserResponse',
    full_name='grid.v1.DeleteUserResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='user',
            full_name='grid.v1.DeleteUserResponse.user',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='user',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=871,
    serialized_end=926,
)

_WATCHUSERSREQUEST = _descriptor.Descriptor(
    name='WatchUsersRequest',
    full_name='grid.v1.WatchUsersRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='resource_version',
            full_name='grid.v1.WatchUsersRequest.resource_version',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='resourceVersion',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='limit',
            full_name='grid.v1.WatchUsersRequest.limit',
            index=1,
            number=2,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='limit',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=928,
    serialized_end=1012,
)

_WATCHUSERSRESPONSE = _descriptor.Descriptor(
    name='WatchUsersResponse',
    full_name='grid.v1.WatchUsersResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='resource_version',
            full_name='grid.v1.WatchUsersResponse.resource_version',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='resourceVersion',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='type',
            full_name='grid.v1.WatchUsersResponse.type',
            index=1,
            number=2,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='type',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='user',
            full_name='grid.v1.WatchUsersResponse.user',
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='user',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1015,
    serialized_end=1158,
)

_STOPUSERWORKLOADSREQUEST = _descriptor.Descriptor(
    name='StopUserWorkloadsRequest',
    full_name='grid.v1.StopUserWorkloadsRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='user_id',
            full_name='grid.v1.StopUserWorkloadsRequest.user_id',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='userId',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1160,
    serialized_end=1211,
)

_STOPUSERWORKLOADSRESPONSE = _descriptor.Descriptor(
    name='StopUserWorkloadsResponse',
    full_name='grid.v1.StopUserWorkloadsResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1213,
    serialized_end=1240,
)

_CREATEUSERREQUEST.fields_by_name['user'].message_type = grid_dot_v1_dot_user__pb2._USER
_CREATEUSERRESPONSE.fields_by_name['user'].message_type = grid_dot_v1_dot_user__pb2._USER
_GETUSERRESPONSE.fields_by_name['user'].message_type = grid_dot_v1_dot_user__pb2._USER
_LISTUSERSRESPONSE.fields_by_name['user'].message_type = grid_dot_v1_dot_user__pb2._USER
_UPDATEUSERREQUEST.fields_by_name['user'].message_type = grid_dot_v1_dot_user__pb2._USER
_UPDATEUSERRESPONSE.fields_by_name['user'].message_type = grid_dot_v1_dot_user__pb2._USER
_UPDATEUSERSTATUSREQUEST.fields_by_name['user'].message_type = grid_dot_v1_dot_user__pb2._USER
_UPDATEUSERSTATUSRESPONSE.fields_by_name['user'].message_type = grid_dot_v1_dot_user__pb2._USER
_DELETEUSERREQUEST.fields_by_name['user'].message_type = grid_dot_v1_dot_user__pb2._USER
_DELETEUSERRESPONSE.fields_by_name['user'].message_type = grid_dot_v1_dot_user__pb2._USER
_WATCHUSERSRESPONSE.fields_by_name['type'].enum_type = grid_dot_v1_dot_metadata__pb2._WATCHEVENTTYPE
_WATCHUSERSRESPONSE.fields_by_name['user'].message_type = grid_dot_v1_dot_user__pb2._USER
DESCRIPTOR.message_types_by_name['CreateUserRequest'] = _CREATEUSERREQUEST
DESCRIPTOR.message_types_by_name['CreateUserResponse'] = _CREATEUSERRESPONSE
DESCRIPTOR.message_types_by_name['GetUserRequest'] = _GETUSERREQUEST
DESCRIPTOR.message_types_by_name['GetUserResponse'] = _GETUSERRESPONSE
DESCRIPTOR.message_types_by_name['ListUsersRequest'] = _LISTUSERSREQUEST
DESCRIPTOR.message_types_by_name['ListUsersResponse'] = _LISTUSERSRESPONSE
DESCRIPTOR.message_types_by_name['UpdateUserRequest'] = _UPDATEUSERREQUEST
DESCRIPTOR.message_types_by_name['UpdateUserResponse'] = _UPDATEUSERRESPONSE
DESCRIPTOR.message_types_by_name['UpdateUserStatusRequest'] = _UPDATEUSERSTATUSREQUEST
DESCRIPTOR.message_types_by_name['UpdateUserStatusResponse'] = _UPDATEUSERSTATUSRESPONSE
DESCRIPTOR.message_types_by_name['DeleteUserRequest'] = _DELETEUSERREQUEST
DESCRIPTOR.message_types_by_name['DeleteUserResponse'] = _DELETEUSERRESPONSE
DESCRIPTOR.message_types_by_name['WatchUsersRequest'] = _WATCHUSERSREQUEST
DESCRIPTOR.message_types_by_name['WatchUsersResponse'] = _WATCHUSERSRESPONSE
DESCRIPTOR.message_types_by_name['StopUserWorkloadsRequest'] = _STOPUSERWORKLOADSREQUEST
DESCRIPTOR.message_types_by_name['StopUserWorkloadsResponse'] = _STOPUSERWORKLOADSRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

CreateUserRequest = _reflection.GeneratedProtocolMessageType(
    'CreateUserRequest',
    (_message.Message, ),
    {
        'DESCRIPTOR': _CREATEUSERREQUEST,
        '__module__': 'grid.v1.user_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.CreateUserRequest)
    }
)
_sym_db.RegisterMessage(CreateUserRequest)

CreateUserResponse = _reflection.GeneratedProtocolMessageType(
    'CreateUserResponse',
    (_message.Message, ),
    {
        'DESCRIPTOR': _CREATEUSERRESPONSE,
        '__module__': 'grid.v1.user_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.CreateUserResponse)
    }
)
_sym_db.RegisterMessage(CreateUserResponse)

GetUserRequest = _reflection.GeneratedProtocolMessageType(
    'GetUserRequest',
    (_message.Message, ),
    {
        'DESCRIPTOR': _GETUSERREQUEST,
        '__module__': 'grid.v1.user_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.GetUserRequest)
    }
)
_sym_db.RegisterMessage(GetUserRequest)

GetUserResponse = _reflection.GeneratedProtocolMessageType(
    'GetUserResponse',
    (_message.Message, ),
    {
        'DESCRIPTOR': _GETUSERRESPONSE,
        '__module__': 'grid.v1.user_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.GetUserResponse)
    }
)
_sym_db.RegisterMessage(GetUserResponse)

ListUsersRequest = _reflection.GeneratedProtocolMessageType(
    'ListUsersRequest',
    (_message.Message, ),
    {
        'DESCRIPTOR': _LISTUSERSREQUEST,
        '__module__': 'grid.v1.user_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.ListUsersRequest)
    }
)
_sym_db.RegisterMessage(ListUsersRequest)

ListUsersResponse = _reflection.GeneratedProtocolMessageType(
    'ListUsersResponse',
    (_message.Message, ),
    {
        'DESCRIPTOR': _LISTUSERSRESPONSE,
        '__module__': 'grid.v1.user_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.ListUsersResponse)
    }
)
_sym_db.RegisterMessage(ListUsersResponse)

UpdateUserRequest = _reflection.GeneratedProtocolMessageType(
    'UpdateUserRequest',
    (_message.Message, ),
    {
        'DESCRIPTOR': _UPDATEUSERREQUEST,
        '__module__': 'grid.v1.user_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.UpdateUserRequest)
    }
)
_sym_db.RegisterMessage(UpdateUserRequest)

UpdateUserResponse = _reflection.GeneratedProtocolMessageType(
    'UpdateUserResponse',
    (_message.Message, ),
    {
        'DESCRIPTOR': _UPDATEUSERRESPONSE,
        '__module__': 'grid.v1.user_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.UpdateUserResponse)
    }
)
_sym_db.RegisterMessage(UpdateUserResponse)

UpdateUserStatusRequest = _reflection.GeneratedProtocolMessageType(
    'UpdateUserStatusRequest',
    (_message.Message, ),
    {
        'DESCRIPTOR': _UPDATEUSERSTATUSREQUEST,
        '__module__': 'grid.v1.user_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.UpdateUserStatusRequest)
    }
)
_sym_db.RegisterMessage(UpdateUserStatusRequest)

UpdateUserStatusResponse = _reflection.GeneratedProtocolMessageType(
    'UpdateUserStatusResponse',
    (_message.Message, ),
    {
        'DESCRIPTOR': _UPDATEUSERSTATUSRESPONSE,
        '__module__': 'grid.v1.user_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.UpdateUserStatusResponse)
    }
)
_sym_db.RegisterMessage(UpdateUserStatusResponse)

DeleteUserRequest = _reflection.GeneratedProtocolMessageType(
    'DeleteUserRequest',
    (_message.Message, ),
    {
        'DESCRIPTOR': _DELETEUSERREQUEST,
        '__module__': 'grid.v1.user_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.DeleteUserRequest)
    }
)
_sym_db.RegisterMessage(DeleteUserRequest)

DeleteUserResponse = _reflection.GeneratedProtocolMessageType(
    'DeleteUserResponse',
    (_message.Message, ),
    {
        'DESCRIPTOR': _DELETEUSERRESPONSE,
        '__module__': 'grid.v1.user_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.DeleteUserResponse)
    }
)
_sym_db.RegisterMessage(DeleteUserResponse)

WatchUsersRequest = _reflection.GeneratedProtocolMessageType(
    'WatchUsersRequest',
    (_message.Message, ),
    {
        'DESCRIPTOR': _WATCHUSERSREQUEST,
        '__module__': 'grid.v1.user_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.WatchUsersRequest)
    }
)
_sym_db.RegisterMessage(WatchUsersRequest)

WatchUsersResponse = _reflection.GeneratedProtocolMessageType(
    'WatchUsersResponse',
    (_message.Message, ),
    {
        'DESCRIPTOR': _WATCHUSERSRESPONSE,
        '__module__': 'grid.v1.user_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.WatchUsersResponse)
    }
)
_sym_db.RegisterMessage(WatchUsersResponse)

StopUserWorkloadsRequest = _reflection.GeneratedProtocolMessageType(
    'StopUserWorkloadsRequest',
    (_message.Message, ),
    {
        'DESCRIPTOR': _STOPUSERWORKLOADSREQUEST,
        '__module__': 'grid.v1.user_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.StopUserWorkloadsRequest)
    }
)
_sym_db.RegisterMessage(StopUserWorkloadsRequest)

StopUserWorkloadsResponse = _reflection.GeneratedProtocolMessageType(
    'StopUserWorkloadsResponse',
    (_message.Message, ),
    {
        'DESCRIPTOR': _STOPUSERWORKLOADSRESPONSE,
        '__module__': 'grid.v1.user_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.StopUserWorkloadsResponse)
    }
)
_sym_db.RegisterMessage(StopUserWorkloadsResponse)

DESCRIPTOR._options = None

_USERSERVICE = _descriptor.ServiceDescriptor(
    name='UserService',
    full_name='grid.v1.UserService',
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=1243,
    serialized_end=1863,
    methods=[
        _descriptor.MethodDescriptor(
            name='CreateUser',
            full_name='grid.v1.UserService.CreateUser',
            index=0,
            containing_service=None,
            input_type=_CREATEUSERREQUEST,
            output_type=_CREATEUSERRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name='GetUser',
            full_name='grid.v1.UserService.GetUser',
            index=1,
            containing_service=None,
            input_type=_GETUSERREQUEST,
            output_type=_GETUSERRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name='ListUsers',
            full_name='grid.v1.UserService.ListUsers',
            index=2,
            containing_service=None,
            input_type=_LISTUSERSREQUEST,
            output_type=_LISTUSERSRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name='UpdateUser',
            full_name='grid.v1.UserService.UpdateUser',
            index=3,
            containing_service=None,
            input_type=_UPDATEUSERREQUEST,
            output_type=_UPDATEUSERRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name='UpdateUserStatus',
            full_name='grid.v1.UserService.UpdateUserStatus',
            index=4,
            containing_service=None,
            input_type=_UPDATEUSERSTATUSREQUEST,
            output_type=_UPDATEUSERSTATUSRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name='DeleteUser',
            full_name='grid.v1.UserService.DeleteUser',
            index=5,
            containing_service=None,
            input_type=_DELETEUSERREQUEST,
            output_type=_DELETEUSERRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name='WatchUsers',
            full_name='grid.v1.UserService.WatchUsers',
            index=6,
            containing_service=None,
            input_type=_WATCHUSERSREQUEST,
            output_type=_WATCHUSERSRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name='StopUserWorkloads',
            full_name='grid.v1.UserService.StopUserWorkloads',
            index=7,
            containing_service=None,
            input_type=_STOPUSERWORKLOADSREQUEST,
            output_type=_STOPUSERWORKLOADSRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
    ]
)
_sym_db.RegisterServiceDescriptor(_USERSERVICE)

DESCRIPTOR.services_by_name['UserService'] = _USERSERVICE

# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from grid.protos.grid.v1 import sshpublickey_service_pb2 as grid_dot_v1_dot_sshpublickey__service__pb2


class SSHPublicKeyServiceStub(object):
    # missing associated documentation comment in .proto file
    pass

    def __init__(self, channel):
        """Constructor.

    Args:
      channel: A grpc.Channel.
    """
        self.CreateSSHPublicKey = channel.unary_unary(
            '/grid.v1.SSHPublicKeyService/CreateSSHPublicKey',
            request_serializer=grid_dot_v1_dot_sshpublickey__service__pb2.CreateSSHPublicKeyRequest.SerializeToString,
            response_deserializer=grid_dot_v1_dot_sshpublickey__service__pb2.CreateSSHPublicKeyResponse.FromString,
        )
        self.GetSSHPublicKey = channel.unary_unary(
            '/grid.v1.SSHPublicKeyService/GetSSHPublicKey',
            request_serializer=grid_dot_v1_dot_sshpublickey__service__pb2.GetSSHPublicKeyRequest.SerializeToString,
            response_deserializer=grid_dot_v1_dot_sshpublickey__service__pb2.GetSSHPublicKeyResponse.FromString,
        )
        self.ListSSHPublicKeys = channel.unary_unary(
            '/grid.v1.SSHPublicKeyService/ListSSHPublicKeys',
            request_serializer=grid_dot_v1_dot_sshpublickey__service__pb2.ListSSHPublicKeysRequest.SerializeToString,
            response_deserializer=grid_dot_v1_dot_sshpublickey__service__pb2.ListSSHPublicKeysResponse.FromString,
        )
        self.UpdateSSHPublicKey = channel.unary_unary(
            '/grid.v1.SSHPublicKeyService/UpdateSSHPublicKey',
            request_serializer=grid_dot_v1_dot_sshpublickey__service__pb2.UpdateSSHPublicKeyRequest.SerializeToString,
            response_deserializer=grid_dot_v1_dot_sshpublickey__service__pb2.UpdateSSHPublicKeyResponse.FromString,
        )
        self.UpdateSSHPublicKeyStatus = channel.unary_unary(
            '/grid.v1.SSHPublicKeyService/UpdateSSHPublicKeyStatus',
            request_serializer=grid_dot_v1_dot_sshpublickey__service__pb2.UpdateSSHPublicKeyStatusRequest.
            SerializeToString,
            response_deserializer=grid_dot_v1_dot_sshpublickey__service__pb2.UpdateSSHPublicKeyStatusResponse.
            FromString,
        )
        self.DeleteSSHPublicKey = channel.unary_unary(
            '/grid.v1.SSHPublicKeyService/DeleteSSHPublicKey',
            request_serializer=grid_dot_v1_dot_sshpublickey__service__pb2.DeleteSSHPublicKeyRequest.SerializeToString,
            response_deserializer=grid_dot_v1_dot_sshpublickey__service__pb2.DeleteSSHPublicKeyResponse.FromString,
        )
        self.WatchSSHPublicKeys = channel.unary_stream(
            '/grid.v1.SSHPublicKeyService/WatchSSHPublicKeys',
            request_serializer=grid_dot_v1_dot_sshpublickey__service__pb2.WatchSSHPublicKeysRequest.SerializeToString,
            response_deserializer=grid_dot_v1_dot_sshpublickey__service__pb2.WatchSSHPublicKeysResponse.FromString,
        )


class SSHPublicKeyServiceServicer(object):
    # missing associated documentation comment in .proto file
    pass

    def CreateSSHPublicKey(self, request, context):
        # missing associated documentation comment in .proto file
        pass
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetSSHPublicKey(self, request, context):
        # missing associated documentation comment in .proto file
        pass
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListSSHPublicKeys(self, request, context):
        # missing associated documentation comment in .proto file
        pass
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateSSHPublicKey(self, request, context):
        # missing associated documentation comment in .proto file
        pass
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateSSHPublicKeyStatus(self, request, context):
        # missing associated documentation comment in .proto file
        pass
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteSSHPublicKey(self, request, context):
        # missing associated documentation comment in .proto file
        pass
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WatchSSHPublicKeys(self, request, context):
        # missing associated documentation comment in .proto file
        pass
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_SSHPublicKeyServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        'CreateSSHPublicKey':
        grpc.unary_unary_rpc_method_handler(
            servicer.CreateSSHPublicKey,
            request_deserializer=grid_dot_v1_dot_sshpublickey__service__pb2.CreateSSHPublicKeyRequest.FromString,
            response_serializer=grid_dot_v1_dot_sshpublickey__service__pb2.CreateSSHPublicKeyResponse.SerializeToString,
        ),
        'GetSSHPublicKey':
        grpc.unary_unary_rpc_method_handler(
            servicer.GetSSHPublicKey,
            request_deserializer=grid_dot_v1_dot_sshpublickey__service__pb2.GetSSHPublicKeyRequest.FromString,
            response_serializer=grid_dot_v1_dot_sshpublickey__service__pb2.GetSSHPublicKeyResponse.SerializeToString,
        ),
        'ListSSHPublicKeys':
        grpc.unary_unary_rpc_method_handler(
            servicer.ListSSHPublicKeys,
            request_deserializer=grid_dot_v1_dot_sshpublickey__service__pb2.ListSSHPublicKeysRequest.FromString,
            response_serializer=grid_dot_v1_dot_sshpublickey__service__pb2.ListSSHPublicKeysResponse.SerializeToString,
        ),
        'UpdateSSHPublicKey':
        grpc.unary_unary_rpc_method_handler(
            servicer.UpdateSSHPublicKey,
            request_deserializer=grid_dot_v1_dot_sshpublickey__service__pb2.UpdateSSHPublicKeyRequest.FromString,
            response_serializer=grid_dot_v1_dot_sshpublickey__service__pb2.UpdateSSHPublicKeyResponse.SerializeToString,
        ),
        'UpdateSSHPublicKeyStatus':
        grpc.unary_unary_rpc_method_handler(
            servicer.UpdateSSHPublicKeyStatus,
            request_deserializer=grid_dot_v1_dot_sshpublickey__service__pb2.UpdateSSHPublicKeyStatusRequest.FromString,
            response_serializer=grid_dot_v1_dot_sshpublickey__service__pb2.UpdateSSHPublicKeyStatusResponse.
            SerializeToString,
        ),
        'DeleteSSHPublicKey':
        grpc.unary_unary_rpc_method_handler(
            servicer.DeleteSSHPublicKey,
            request_deserializer=grid_dot_v1_dot_sshpublickey__service__pb2.DeleteSSHPublicKeyRequest.FromString,
            response_serializer=grid_dot_v1_dot_sshpublickey__service__pb2.DeleteSSHPublicKeyResponse.SerializeToString,
        ),
        'WatchSSHPublicKeys':
        grpc.unary_stream_rpc_method_handler(
            servicer.WatchSSHPublicKeys,
            request_deserializer=grid_dot_v1_dot_sshpublickey__service__pb2.WatchSSHPublicKeysRequest.FromString,
            response_serializer=grid_dot_v1_dot_sshpublickey__service__pb2.WatchSSHPublicKeysResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler('grid.v1.SSHPublicKeyService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler, ))



# coding: utf-8
"""
    external/v1/external_session_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

from grid.openapi.configuration import Configuration


class V1KubernetesClusterDriver(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'aws': 'V1AWSClusterDriverSpec',
        'azure': 'V1AzureClusterDriverSpec',
        'kubeconfig': 'V1ExternalKubeconfig',
        'namespace': 'str',
        'root_domain_name': 'str'
    }

    attribute_map = {
        'aws': 'aws',
        'azure': 'azure',
        'kubeconfig': 'kubeconfig',
        'namespace': 'namespace',
        'root_domain_name': 'rootDomainName'
    }

    def __init__(
        self,
        aws=None,
        azure=None,
        kubeconfig=None,
        namespace=None,
        root_domain_name=None,
        _configuration=None
    ):  # noqa: E501
        """V1KubernetesClusterDriver - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._aws = None
        self._azure = None
        self._kubeconfig = None
        self._namespace = None
        self._root_domain_name = None
        self.discriminator = None

        if aws is not None:
            self.aws = aws
        if azure is not None:
            self.azure = azure
        if kubeconfig is not None:
            self.kubeconfig = kubeconfig
        if namespace is not None:
            self.namespace = namespace
        if root_domain_name is not None:
            self.root_domain_name = root_domain_name

    @property
    def aws(self):
        """Gets the aws of this V1KubernetesClusterDriver.  # noqa: E501


        :return: The aws of this V1KubernetesClusterDriver.  # noqa: E501
        :rtype: V1AWSClusterDriverSpec
        """
        return self._aws

    @aws.setter
    def aws(self, aws):
        """Sets the aws of this V1KubernetesClusterDriver.


        :param aws: The aws of this V1KubernetesClusterDriver.  # noqa: E501
        :type: V1AWSClusterDriverSpec
        """

        self._aws = aws

    @property
    def azure(self):
        """Gets the azure of this V1KubernetesClusterDriver.  # noqa: E501


        :return: The azure of this V1KubernetesClusterDriver.  # noqa: E501
        :rtype: V1AzureClusterDriverSpec
        """
        return self._azure

    @azure.setter
    def azure(self, azure):
        """Sets the azure of this V1KubernetesClusterDriver.


        :param azure: The azure of this V1KubernetesClusterDriver.  # noqa: E501
        :type: V1AzureClusterDriverSpec
        """

        self._azure = azure

    @property
    def kubeconfig(self):
        """Gets the kubeconfig of this V1KubernetesClusterDriver.  # noqa: E501


        :return: The kubeconfig of this V1KubernetesClusterDriver.  # noqa: E501
        :rtype: V1ExternalKubeconfig
        """
        return self._kubeconfig

    @kubeconfig.setter
    def kubeconfig(self, kubeconfig):
        """Sets the kubeconfig of this V1KubernetesClusterDriver.


        :param kubeconfig: The kubeconfig of this V1KubernetesClusterDriver.  # noqa: E501
        :type: V1ExternalKubeconfig
        """

        self._kubeconfig = kubeconfig

    @property
    def namespace(self):
        """Gets the namespace of this V1KubernetesClusterDriver.  # noqa: E501


        :return: The namespace of this V1KubernetesClusterDriver.  # noqa: E501
        :rtype: str
        """
        return self._namespace

    @namespace.setter
    def namespace(self, namespace):
        """Sets the namespace of this V1KubernetesClusterDriver.


        :param namespace: The namespace of this V1KubernetesClusterDriver.  # noqa: E501
        :type: str
        """

        self._namespace = namespace

    @property
    def root_domain_name(self):
        """Gets the root_domain_name of this V1KubernetesClusterDriver.  # noqa: E501


        :return: The root_domain_name of this V1KubernetesClusterDriver.  # noqa: E501
        :rtype: str
        """
        return self._root_domain_name

    @root_domain_name.setter
    def root_domain_name(self, root_domain_name):
        """Sets the root_domain_name of this V1KubernetesClusterDriver.


        :param root_domain_name: The root_domain_name of this V1KubernetesClusterDriver.  # noqa: E501
        :type: str
        """

        self._root_domain_name = root_domain_name

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict") else item, value.items()
                    )
                )
            else:
                result[attr] = value
        if issubclass(V1KubernetesClusterDriver, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1KubernetesClusterDriver):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1KubernetesClusterDriver):
            return True

        return self.to_dict() != other.to_dict()



"""TODO(rlizzo): THIS FILE IS PART OF PROJECT FEYNMAN
"""
import hashlib
from pathlib import Path
from typing import Set


def dirhash(path: Path, algorithm: str = "blake2", ignore: Set[str] = {}, chunk_num_blocks: int = 128) -> str:
    """
    Hashes the contents of an entire directory, excluding known glob patterns.

    Parameters
    ----------
    path: Path
        Path to directory.
    algorithm: str, default "blake2"
        Algorithm to hash contents. "blake2" is set by default because it
        is faster than "md5". [1]
    ignore: Set[str], default {}
        Set of glob patterns to ignore.
    chunk_num_blocks: int, default 128
        Block size to user when iterating over file chunks.

    References
    ----------
    [1] https://crypto.stackexchange.com/questions/70101/blake2-vs-md5-for-checksum-file-integrity
    [2] https://stackoverflow.com/questions/1131220/get-md5-hash-of-big-files-in-python
    """
    # validate input
    if algorithm == "blake2":
        h = hashlib.blake2b(digest_size=20)
    elif algorithm == "md5":
        h = hashlib.md5()
    else:
        raise ValueError(f"Algorithm {algorithm} not supported")

    # first get entire set of files
    include: Set[Path] = set()
    for file in path.glob("**/*"):
        include.add(file)

    # then remove all files that match a glob pattern
    exclude: Set[Path] = set()
    for pattern in ignore:
        for file in path.glob(pattern):
            exclude.add(file)

    # edit include set
    include = include - exclude

    # calculate hash for all files
    for file in include:
        if not file.is_file():
            continue
        with file.open("rb") as f:
            for chunk in iter(lambda: f.read(chunk_num_blocks * h.block_size), b''):
                h.update(chunk)

    return h.hexdigest()



from functools import lru_cache, wraps
import json
from typing import Dict, Iterator, Optional
from urllib import parse

from gql import Client, gql
from gql.transport.exceptions import TransportProtocolError
from gql.transport.requests import RequestsHTTPTransport
from gql.transport.websockets import WebsocketsTransport
import grpc
import requests
from websockets.exceptions import ConnectionClosedError, ConnectionClosedOK

from grid.metadata import __version__
from grid.sdk.auth import Credentials, GrpcAuth


def humanize_gql_exceptions(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)

        # runtime errors are generated if gql returns with an "error" message
        except RuntimeError as e:
            raise e from None

        except requests.exceptions.ConnectionError:
            raise ConnectionError(
                f'Grid is unreachable. Can you access https://platform.grid.ai ? '
                f'If not, please try again later.'
            ) from None

        except requests.exceptions.HTTPError as e:
            if e.response.status_code == 401:
                raise ConnectionRefusedError('Not authorized. Did you login?') from None
            if e.response.status_code == 500:
                raise ConnectionError('Grid is having issues. Please again later.') from None
            raise ConnectionError(f'We encountered an unknown error. Please try again later. {type(e).__name__}: {e}')

        except requests.exceptions.Timeout:
            raise ConnectionError('The grid platform could not be reached. Are you online?') from None
        except TimeoutError:
            raise RuntimeError(f"Timeout waiting for messages exceeded") from None

        except TransportProtocolError:
            raise ConnectionRefusedError('You are not authorized to use Grid. Did you login?') from None

        # If connection is suddenly closed, indicate that a known error happened.
        except ConnectionClosedError as e:
            raise ConnectionError(f"Interacting with closed connection. Error: {type(e).__name__}") from None
        except ConnectionClosedOK as e:
            raise ConnectionError(
                f"Interacting with properly closed connection. No new "
                f"messages available. Error {type(e).__name__}"
            ) from None

        except Exception as e:
            raise RuntimeError(f"Unhandled error. Please report to the grid team. {type(e).__name__}: {str(e)}")

    return wrapper


@lru_cache(maxsize=None)
@humanize_gql_exceptions
def gql_client(
    url: str,
    creds: Credentials,
    websocket: bool = False,
    *,
    timeout: int = 10,
    headers: Optional[Dict[str, str]] = None
) -> Client:
    """Only create a client instance if there isn't one.

    TODO(rlizzo): re-enable check_version_compatability when versioning is fixed
    """
    if headers is None:
        headers = {}

    headers.update({
        'Content-type': 'application/json',
        'User-Agent': f'grid-api-{__version__}',
        "X-Grid-User": creds.user_id,
        "X-Grid-Key": creds.api_key,
    })

    if websocket:
        _url = url.replace('http://', 'ws://')
        _url = _url.replace('https://', 'wss://')
        _url = _url.replace('graphql', 'subscriptions')
        transport = WebsocketsTransport(url=_url, init_payload=headers)
    else:
        transport = RequestsHTTPTransport(url=url, use_json=True, headers=headers, timeout=timeout, retries=3)

    res = Client(transport=transport, fetch_schema_from_transport=True)
    return res


@humanize_gql_exceptions
def gql_execute(client: Client, query: str, **kwargs) -> dict:
    """Executes a GraphQL query against Grid's API.
    """
    res = client.execute(gql(query), variable_values=kwargs)
    if 'error' in res:
        raise RuntimeError(json.dumps(res['error']))
    return res


@humanize_gql_exceptions
def gql_subscribe(client: Client, query: str, **kwargs) -> Iterator[dict]:
    """Streams data from a GraphQL subscription.

    TODO(rlizzo):
        * The qgl library has issues when running this function in jupyterlab
          due to the ipykernel's  eventloop. It sees this eventloop running
          in the same thread and therefore require ``await``-ing an analogous
          call to ``.async_subscribe``. Obviously transforming async -> sync
          code paths aren't fun, so we need to wrap this function in a method
          which determines current context and can run an analogous coroutine
          in some event loop (i'm not sure if it should be ipykernel's - which
          might be blocked on the jupyterlab code cell execution? - or a new
          event loop...)
          _
          The point is that neither the rest of the sdk, nor the user, should
          care that we have to make one async call here. NO UPSTREAM CODE PATH
          CHANGES are to be converted to async. It is a massive pain for both
          developers and end users alike. SERIOUSLY. DON'T DO IT.
    """
    stream = client.subscribe(gql(query), variable_values=kwargs)
    for element in stream:
        yield element


async def agrpc_channel(url: str, creds: Credentials) -> grpc.aio.Channel:
    authentication = GrpcAuth(creds)
    call_credentials = grpc.metadata_call_credentials(authentication)
    channel_creds = grpc.ssl_channel_credentials()

    url = parse.urlparse(url)
    channel = grpc.aio.secure_channel(
        f"{url.hostname}:{url.port or 443}", grpc.composite_channel_credentials(
            channel_creds,
            call_credentials,
        )
    )
    return channel


def grpc_channel(url: str, creds: Credentials) -> grpc.Channel:
    authentication = GrpcAuth(creds)
    call_credentials = grpc.metadata_call_credentials(authentication)
    channel_creds = grpc.ssl_channel_credentials()

    url = parse.urlparse(url)
    channel = grpc.secure_channel(
        f"{url.hostname}:{url.port or 443}", grpc.composite_channel_credentials(
            channel_creds,
            call_credentials,
        )
    )
    return channel



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grid/v1/session.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import enum_type_wrapper

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2

from grid.protos.grid.v1 import datastore_pb2 as grid_dot_v1_dot_datastore__pb2
from grid.protos.grid.v1 import metadata_pb2 as grid_dot_v1_dot_metadata__pb2
from grid.protos.grid.v1 import resource_pb2 as grid_dot_v1_dot_resource__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name='grid/v1/session.proto',
    package='grid.v1',
    syntax='proto3',
    serialized_options=b'Z0github.com/gridai/grid/grid-backend/apis/grid/v1',
    create_key=_descriptor._internal_create_key,
    serialized_pb=
    b'\n\x15grid/v1/session.proto\x12\x07grid.v1\x1a\x16grid/v1/metadata.proto\x1a\x17grid/v1/datastore.proto\x1a\x16grid/v1/resource.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1fgoogle/api/field_behavior.proto\"\x92\x01\n\x07Session\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x11.grid.v1.MetadataR\x08metadata\x12(\n\x04spec\x18\x02 \x01(\x0b\x32\x14.grid.v1.SessionSpecR\x04spec\x12.\n\x06status\x18\x03 \x01(\x0b\x32\x16.grid.v1.SessionStatusR\x06status\"\xe3\x02\n\x0bSessionSpec\x12\x1d\n\ncluster_id\x18\x01 \x01(\tR\tclusterId\x12\x1d\n\x07user_id\x18\x02 \x01(\tB\x04\xe2\x41\x01\x05R\x06userId\x12:\n\rdesired_state\x18\x03 \x01(\x0e\x32\x15.grid.v1.SessionStateR\x0c\x64\x65siredState\x12#\n\rinstance_type\x18\x04 \x01(\tR\x0cinstanceType\x12\x30\n\tresources\x18\x05 \x01(\x0b\x32\x12.grid.v1.ResourcesR\tresources\x12\x37\n\ndatastores\x18\x06 \x03(\x0b\x32\x17.grid.v1.DatastoreInputR\ndatastores\x12/\n\x10pause_automation\x18\x07 \x01(\x08\x42\x04\xe2\x41\x01\x03R\x0fpauseAutomation\x12\x19\n\x08use_spot\x18\x08 \x01(\x08R\x07useSpot\"\xa3\x04\n\rSessionStatus\x12+\n\x05phase\x18\x01 \x01(\x0e\x32\x15.grid.v1.SessionStateR\x05phase\x12\x17\n\x07ssh_url\x18\x02 \x01(\tR\x06sshUrl\x12&\n\x0fjupyter_lab_url\x18\x03 \x01(\tR\rjupyterLabUrl\x12\x33\n\x16total_run_time_seconds\x18\x04 \x01(\x04R\x13totalRunTimeSeconds\x12\x43\n\x0fstart_timestamp\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0estartTimestamp\x12\x41\n\x0estop_timestamp\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\rstopTimestamp\x12K\n\x13\x64\x65stroyed_timestamp\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x12\x64\x65stroyedTimestamp\x12n\n&last_state_status_transition_timestamp\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\"lastStateStatusTransitionTimestamp\x12*\n\x11jupyter_lab_token\x18\t \x01(\tR\x0fjupyterLabToken*\x98\x01\n\x0cSessionState\x12\x1d\n\x19SESSION_STATE_UNSPECIFIED\x10\x00\x12\x19\n\x15SESSION_STATE_PENDING\x10\x01\x12\x19\n\x15SESSION_STATE_RUNNING\x10\x02\x12\x18\n\x14SESSION_STATE_PAUSED\x10\x03\x12\x19\n\x15SESSION_STATE_DELETED\x10\x04\x42\x32Z0github.com/gridai/grid/grid-backend/apis/grid/v1b\x06proto3',
    dependencies=[
        grid_dot_v1_dot_metadata__pb2.DESCRIPTOR,
        grid_dot_v1_dot_datastore__pb2.DESCRIPTOR,
        grid_dot_v1_dot_resource__pb2.DESCRIPTOR,
        google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,
        google_dot_api_dot_field__behavior__pb2.DESCRIPTOR,
    ]
)

_SESSIONSTATE = _descriptor.EnumDescriptor(
    name='SessionState',
    full_name='grid.v1.SessionState',
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name='SESSION_STATE_UNSPECIFIED',
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name='SESSION_STATE_PENDING',
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name='SESSION_STATE_RUNNING',
            index=2,
            number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name='SESSION_STATE_PAUSED',
            index=3,
            number=3,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name='SESSION_STATE_DELETED',
            index=4,
            number=4,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=1231,
    serialized_end=1383,
)
_sym_db.RegisterEnumDescriptor(_SESSIONSTATE)

SessionState = enum_type_wrapper.EnumTypeWrapper(_SESSIONSTATE)
SESSION_STATE_UNSPECIFIED = 0
SESSION_STATE_PENDING = 1
SESSION_STATE_RUNNING = 2
SESSION_STATE_PAUSED = 3
SESSION_STATE_DELETED = 4

_SESSION = _descriptor.Descriptor(
    name='Session',
    full_name='grid.v1.Session',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='metadata',
            full_name='grid.v1.Session.metadata',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='metadata',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='spec',
            full_name='grid.v1.Session.spec',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='spec',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='status',
            full_name='grid.v1.Session.status',
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='status',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=174,
    serialized_end=320,
)

_SESSIONSPEC = _descriptor.Descriptor(
    name='SessionSpec',
    full_name='grid.v1.SessionSpec',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='cluster_id',
            full_name='grid.v1.SessionSpec.cluster_id',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='clusterId',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='user_id',
            full_name='grid.v1.SessionSpec.user_id',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=b'\342A\001\005',
            json_name='userId',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='desired_state',
            full_name='grid.v1.SessionSpec.desired_state',
            index=2,
            number=3,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='desiredState',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='instance_type',
            full_name='grid.v1.SessionSpec.instance_type',
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='instanceType',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='resources',
            full_name='grid.v1.SessionSpec.resources',
            index=4,
            number=5,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='resources',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='datastores',
            full_name='grid.v1.SessionSpec.datastores',
            index=5,
            number=6,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='datastores',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='pause_automation',
            full_name='grid.v1.SessionSpec.pause_automation',
            index=6,
            number=7,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=b'\342A\001\003',
            json_name='pauseAutomation',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='use_spot',
            full_name='grid.v1.SessionSpec.use_spot',
            index=7,
            number=8,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='useSpot',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=323,
    serialized_end=678,
)

_SESSIONSTATUS = _descriptor.Descriptor(
    name='SessionStatus',
    full_name='grid.v1.SessionStatus',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='phase',
            full_name='grid.v1.SessionStatus.phase',
            index=0,
            number=1,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='phase',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='ssh_url',
            full_name='grid.v1.SessionStatus.ssh_url',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='sshUrl',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='jupyter_lab_url',
            full_name='grid.v1.SessionStatus.jupyter_lab_url',
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='jupyterLabUrl',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='total_run_time_seconds',
            full_name='grid.v1.SessionStatus.total_run_time_seconds',
            index=3,
            number=4,
            type=4,
            cpp_type=4,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='totalRunTimeSeconds',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='start_timestamp',
            full_name='grid.v1.SessionStatus.start_timestamp',
            index=4,
            number=5,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='startTimestamp',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='stop_timestamp',
            full_name='grid.v1.SessionStatus.stop_timestamp',
            index=5,
            number=6,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='stopTimestamp',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='destroyed_timestamp',
            full_name='grid.v1.SessionStatus.destroyed_timestamp',
            index=6,
            number=7,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='destroyedTimestamp',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='last_state_status_transition_timestamp',
            full_name='grid.v1.SessionStatus.last_state_status_transition_timestamp',
            index=7,
            number=8,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='lastStateStatusTransitionTimestamp',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='jupyter_lab_token',
            full_name='grid.v1.SessionStatus.jupyter_lab_token',
            index=8,
            number=9,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='jupyterLabToken',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=681,
    serialized_end=1228,
)

_SESSION.fields_by_name['metadata'].message_type = grid_dot_v1_dot_metadata__pb2._METADATA
_SESSION.fields_by_name['spec'].message_type = _SESSIONSPEC
_SESSION.fields_by_name['status'].message_type = _SESSIONSTATUS
_SESSIONSPEC.fields_by_name['desired_state'].enum_type = _SESSIONSTATE
_SESSIONSPEC.fields_by_name['resources'].message_type = grid_dot_v1_dot_resource__pb2._RESOURCES
_SESSIONSPEC.fields_by_name['datastores'].message_type = grid_dot_v1_dot_datastore__pb2._DATASTOREINPUT
_SESSIONSTATUS.fields_by_name['phase'].enum_type = _SESSIONSTATE
_SESSIONSTATUS.fields_by_name['start_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_SESSIONSTATUS.fields_by_name['stop_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_SESSIONSTATUS.fields_by_name['destroyed_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_SESSIONSTATUS.fields_by_name['last_state_status_transition_timestamp'
                              ].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
DESCRIPTOR.message_types_by_name['Session'] = _SESSION
DESCRIPTOR.message_types_by_name['SessionSpec'] = _SESSIONSPEC
DESCRIPTOR.message_types_by_name['SessionStatus'] = _SESSIONSTATUS
DESCRIPTOR.enum_types_by_name['SessionState'] = _SESSIONSTATE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Session = _reflection.GeneratedProtocolMessageType(
    'Session',
    (_message.Message, ),
    {
        'DESCRIPTOR': _SESSION,
        '__module__': 'grid.v1.session_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.Session)
    }
)
_sym_db.RegisterMessage(Session)

SessionSpec = _reflection.GeneratedProtocolMessageType(
    'SessionSpec',
    (_message.Message, ),
    {
        'DESCRIPTOR': _SESSIONSPEC,
        '__module__': 'grid.v1.session_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.SessionSpec)
    }
)
_sym_db.RegisterMessage(SessionSpec)

SessionStatus = _reflection.GeneratedProtocolMessageType(
    'SessionStatus',
    (_message.Message, ),
    {
        'DESCRIPTOR': _SESSIONSTATUS,
        '__module__': 'grid.v1.session_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.SessionStatus)
    }
)
_sym_db.RegisterMessage(SessionStatus)

DESCRIPTOR._options = None
_SESSIONSPEC.fields_by_name['user_id']._options = None
_SESSIONSPEC.fields_by_name['pause_automation']._options = None
# @@protoc_insertion_point(module_scope)



"""TODO(rlizzo): THIS FILE IS PART OF PROJECT FEYNMAN

Uploader that uploads files into Cloud storage
"""

from concurrent.futures import as_completed, ThreadPoolExecutor
from typing import AnyStr, Dict, List

import requests
from requests.adapters import HTTPAdapter
from rich.progress import BarColumn, Progress, TextColumn
from urllib3.util.retry import Retry


class UploadProgressCallback:
    """
    This class provides a interface for notifying upload progress
    """
    @staticmethod
    def upload_part_completed(part: int, etag: str):
        return NotImplemented


class S3Uploader:
    """
    This class uploads a source file with presigned urls to S3

    Attributes
    ----------
    source_file: str
        Source file to upload
    presigned_urls: Dict[int, str]
        Presigned urls dictionary, with key as part number and values as urls
    already_uploaded_parts: List[int]
        List of parts that have already been uploaded
    workers: int
        Amount of workers to upload parts
    retries: int
        Amount of retries when requests encounters an error
    backoff_factor: float
        A backoff factor to apply between attempts after the second try
    total_size: int
        Size of all files to upload
    name: str
        Name of this upload to display progress
    progress_callback: UploadProgressCallback
        Callback for notifying upload progress
    """
    workers: int = 8
    retries: int = 10000
    backoff_factor: float = 1.

    progress = Progress(
        TextColumn("[bold blue]{task.description}", justify="left"), BarColumn(bar_width=None),
        "[self.progress.percentage]{task.percentage:>3.1f}%"
    )

    def __init__(
        self,
        presigned_urls: Dict[int, str],
        already_uploaded_parts: List[int],
        source_file: str,
        total_size: int,
        split_size: int,
        name: str,
        progress_callback: UploadProgressCallback = None
    ):
        self.presigned_urls = presigned_urls
        self.already_uploaded_parts = already_uploaded_parts
        self.source_file = source_file
        self.total_size = total_size
        self.split_size = split_size
        self.name = name
        self.progress_callback = progress_callback

    @staticmethod
    def upload_s3_data(url: str, data: bytes, retries: int, backoff_factor: float) -> str:
        """
        Send data to s3 url

        Parameters
        ----------
        url: str
            S3 url string to send data to
        data: bytes
             Bytes of data to send to S3
        retries: int
            Amount of retries
        backoff_factor: float
            Retry backoff factor

        Returns
        -------
        str
            ETag from response
        """
        s = requests.Session()
        retries = Retry(total=retries, backoff_factor=backoff_factor, status_forcelist=[500, 502, 503, 504])
        s.mount('http://', HTTPAdapter(max_retries=retries))
        response = s.put(url, data=data)
        if 'ETag' not in response.headers:
            raise ValueError(f"Unexpected response from S3, response: {response.content}")

        return response.headers['ETag']

    def _get_size(self, part_path: str, part: int) -> AnyStr:
        """
        Returns the size of a specific part.

        Parameters
        ----------
        part_path: str
            Path to part file
        part: int
            Part number
        Returns
        -------
        AnyStr
            Part size
        """
        with open(part_path, 'rb') as f:
            pos = self.split_size * (part - 1)
            f.seek(pos)
            data = f.read(self.split_size)

        return data

    def _progress_already_uploaded(self, task_id: str) -> None:
        """
        Moves the progress bar to account for previously
        uploaded parts so that it shows consistent and
        recoverable progress when resuming the upload.

        Parameters
        ----------
        task_id: str
            Task id for tracking progress
        """
        for part in self.already_uploaded_parts:
            data = self._get_size(part_path=self.source_file, part=part)

            self.progress.update(task_id, advance=len(data))

    def _upload_part(self, url: str, part_path: str, part: int, task_id: str) -> None:
        """
        Upload part of the data file with presigned url

        Parameters
        ----------
        url: str
            Presigned url to upload to S3
        part_path: str
            Path to part file
        part: int
            Part number
        task_id: str
            Task id for tracking progress
        """
        with open(part_path, 'rb') as f:
            pos = self.split_size * (part - 1)
            f.seek(pos)
            data = f.read(self.split_size)

        etag = self.upload_s3_data(url, data, self.retries, self.backoff_factor)
        self.progress.update(task_id, advance=len(data))
        if self.progress_callback:
            self.progress_callback.upload_part_completed(part=part, etag=etag)

    def upload(self) -> None:
        """
        Upload files from source dir into target path in S3
        """
        task_id = self.progress.add_task("upload", filename=self.name, total=self.total_size)
        with ThreadPoolExecutor(max_workers=self.workers) as pool:
            try:
                self.progress.start()
                futures = []

                if self.already_uploaded_parts:
                    f = pool.submit(self._progress_already_uploaded, task_id)
                    futures.append(f)

                    for future in as_completed(futures):
                        future.result()

                for part, url in self.presigned_urls.items():
                    f = pool.submit(self._upload_part, url, self.source_file, part, task_id)
                    futures.append(f)

                    for future in as_completed(futures):
                        future.result()
            finally:
                self.progress.stop()



# coding: utf-8

# flake8: noqa
"""
    external/v1/external_session_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

from grid.openapi.models.body1 import Body1

# import models into model package
from grid.openapi.models.body import Body
from grid.openapi.models.externalv1_cluster import Externalv1Cluster
from grid.openapi.models.externalv1_create_cluster_request import Externalv1CreateClusterRequest
from grid.openapi.models.externalv1_create_cluster_response import Externalv1CreateClusterResponse
from grid.openapi.models.externalv1_create_session_request import Externalv1CreateSessionRequest
from grid.openapi.models.externalv1_create_session_response import Externalv1CreateSessionResponse
from grid.openapi.models.externalv1_delete_cluster_response import Externalv1DeleteClusterResponse
from grid.openapi.models.externalv1_delete_session_response import Externalv1DeleteSessionResponse
from grid.openapi.models.externalv1_get_session_response import Externalv1GetSessionResponse
from grid.openapi.models.externalv1_list_clusters_response import Externalv1ListClustersResponse
from grid.openapi.models.externalv1_list_sessions_response import Externalv1ListSessionsResponse
from grid.openapi.models.externalv1_session import Externalv1Session
from grid.openapi.models.externalv1_update_cluster_response import Externalv1UpdateClusterResponse
from grid.openapi.models.externalv1_update_session_response import Externalv1UpdateSessionResponse
from grid.openapi.models.protobuf_any import ProtobufAny
from grid.openapi.models.rpc_status import RpcStatus
from grid.openapi.models.v1_aws_cluster_driver_spec import V1AWSClusterDriverSpec
from grid.openapi.models.v1_azure_cluster_driver_spec import V1AzureClusterDriverSpec
from grid.openapi.models.v1_azure_cluster_driver_status import V1AzureClusterDriverStatus
from grid.openapi.models.v1_cluster_driver import V1ClusterDriver
from grid.openapi.models.v1_cluster_driver_status import V1ClusterDriverStatus
from grid.openapi.models.v1_cluster_spec import V1ClusterSpec
from grid.openapi.models.v1_cluster_state import V1ClusterState
from grid.openapi.models.v1_cluster_status import V1ClusterStatus
from grid.openapi.models.v1_cluster_type import V1ClusterType
from grid.openapi.models.v1_create_saml_organization_request import V1CreateSamlOrganizationRequest
from grid.openapi.models.v1_create_saml_organization_response import V1CreateSamlOrganizationResponse
from grid.openapi.models.v1_datastore_input import V1DatastoreInput
from grid.openapi.models.v1_delete_saml_organization_response import V1DeleteSamlOrganizationResponse
from grid.openapi.models.v1_eks_custer_driver_status import V1EKSCusterDriverStatus
from grid.openapi.models.v1_external_kubeconfig import V1ExternalKubeconfig
from grid.openapi.models.v1_instance_spec import V1InstanceSpec
from grid.openapi.models.v1_instance_type import V1InstanceType
from grid.openapi.models.v1_kubernetes_cluster_driver import V1KubernetesClusterDriver
from grid.openapi.models.v1_kubernetes_cluster_status import V1KubernetesClusterStatus
from grid.openapi.models.v1_list_cluster_instance_types_response import V1ListClusterInstanceTypesResponse
from grid.openapi.models.v1_list_saml_organizations_response import V1ListSamlOrganizationsResponse
from grid.openapi.models.v1_login_request import V1LoginRequest
from grid.openapi.models.v1_login_response import V1LoginResponse
from grid.openapi.models.v1_logout_request import V1LogoutRequest
from grid.openapi.models.v1_logout_response import V1LogoutResponse
from grid.openapi.models.v1_refresh_request import V1RefreshRequest
from grid.openapi.models.v1_refresh_response import V1RefreshResponse
from grid.openapi.models.v1_resources import V1Resources
from grid.openapi.models.v1_saml_organization import V1SamlOrganization
from grid.openapi.models.v1_saml_organization_status import V1SamlOrganizationStatus
from grid.openapi.models.v1_session_spec import V1SessionSpec
from grid.openapi.models.v1_session_state import V1SessionState
from grid.openapi.models.v1_session_status import V1SessionStatus
from grid.openapi.models.v1_slurm_cluster_driver import V1SlurmClusterDriver
from grid.openapi.models.v1_slurm_cluster_status import V1SlurmClusterStatus



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from grid.protos.grid.v1 import session_service_pb2 as grid_dot_v1_dot_session__service__pb2


class SessionServiceStub(object):
    # missing associated documentation comment in .proto file
    pass

    def __init__(self, channel):
        """Constructor.

    Args:
      channel: A grpc.Channel.
    """
        self.CreateSession = channel.unary_unary(
            '/grid.v1.SessionService/CreateSession',
            request_serializer=grid_dot_v1_dot_session__service__pb2.CreateSessionRequest.SerializeToString,
            response_deserializer=grid_dot_v1_dot_session__service__pb2.CreateSessionResponse.FromString,
        )
        self.GetSession = channel.unary_unary(
            '/grid.v1.SessionService/GetSession',
            request_serializer=grid_dot_v1_dot_session__service__pb2.GetSessionRequest.SerializeToString,
            response_deserializer=grid_dot_v1_dot_session__service__pb2.GetSessionResponse.FromString,
        )
        self.ListSessions = channel.unary_unary(
            '/grid.v1.SessionService/ListSessions',
            request_serializer=grid_dot_v1_dot_session__service__pb2.ListSessionsRequest.SerializeToString,
            response_deserializer=grid_dot_v1_dot_session__service__pb2.ListSessionsResponse.FromString,
        )
        self.UpdateSession = channel.unary_unary(
            '/grid.v1.SessionService/UpdateSession',
            request_serializer=grid_dot_v1_dot_session__service__pb2.UpdateSessionRequest.SerializeToString,
            response_deserializer=grid_dot_v1_dot_session__service__pb2.UpdateSessionResponse.FromString,
        )
        self.UpdateSessionStatus = channel.unary_unary(
            '/grid.v1.SessionService/UpdateSessionStatus',
            request_serializer=grid_dot_v1_dot_session__service__pb2.UpdateSessionStatusRequest.SerializeToString,
            response_deserializer=grid_dot_v1_dot_session__service__pb2.UpdateSessionStatusResponse.FromString,
        )
        self.DeleteSession = channel.unary_unary(
            '/grid.v1.SessionService/DeleteSession',
            request_serializer=grid_dot_v1_dot_session__service__pb2.DeleteSessionRequest.SerializeToString,
            response_deserializer=grid_dot_v1_dot_session__service__pb2.DeleteSessionResponse.FromString,
        )
        self.WatchSessions = channel.unary_stream(
            '/grid.v1.SessionService/WatchSessions',
            request_serializer=grid_dot_v1_dot_session__service__pb2.WatchSessionsRequest.SerializeToString,
            response_deserializer=grid_dot_v1_dot_session__service__pb2.WatchSessionsResponse.FromString,
        )


class SessionServiceServicer(object):
    # missing associated documentation comment in .proto file
    pass

    def CreateSession(self, request, context):
        # missing associated documentation comment in .proto file
        pass
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetSession(self, request, context):
        # missing associated documentation comment in .proto file
        pass
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListSessions(self, request, context):
        # missing associated documentation comment in .proto file
        pass
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateSession(self, request, context):
        # missing associated documentation comment in .proto file
        pass
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateSessionStatus(self, request, context):
        # missing associated documentation comment in .proto file
        pass
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteSession(self, request, context):
        # missing associated documentation comment in .proto file
        pass
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WatchSessions(self, request, context):
        # missing associated documentation comment in .proto file
        pass
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_SessionServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        'CreateSession':
        grpc.unary_unary_rpc_method_handler(
            servicer.CreateSession,
            request_deserializer=grid_dot_v1_dot_session__service__pb2.CreateSessionRequest.FromString,
            response_serializer=grid_dot_v1_dot_session__service__pb2.CreateSessionResponse.SerializeToString,
        ),
        'GetSession':
        grpc.unary_unary_rpc_method_handler(
            servicer.GetSession,
            request_deserializer=grid_dot_v1_dot_session__service__pb2.GetSessionRequest.FromString,
            response_serializer=grid_dot_v1_dot_session__service__pb2.GetSessionResponse.SerializeToString,
        ),
        'ListSessions':
        grpc.unary_unary_rpc_method_handler(
            servicer.ListSessions,
            request_deserializer=grid_dot_v1_dot_session__service__pb2.ListSessionsRequest.FromString,
            response_serializer=grid_dot_v1_dot_session__service__pb2.ListSessionsResponse.SerializeToString,
        ),
        'UpdateSession':
        grpc.unary_unary_rpc_method_handler(
            servicer.UpdateSession,
            request_deserializer=grid_dot_v1_dot_session__service__pb2.UpdateSessionRequest.FromString,
            response_serializer=grid_dot_v1_dot_session__service__pb2.UpdateSessionResponse.SerializeToString,
        ),
        'UpdateSessionStatus':
        grpc.unary_unary_rpc_method_handler(
            servicer.UpdateSessionStatus,
            request_deserializer=grid_dot_v1_dot_session__service__pb2.UpdateSessionStatusRequest.FromString,
            response_serializer=grid_dot_v1_dot_session__service__pb2.UpdateSessionStatusResponse.SerializeToString,
        ),
        'DeleteSession':
        grpc.unary_unary_rpc_method_handler(
            servicer.DeleteSession,
            request_deserializer=grid_dot_v1_dot_session__service__pb2.DeleteSessionRequest.FromString,
            response_serializer=grid_dot_v1_dot_session__service__pb2.DeleteSessionResponse.SerializeToString,
        ),
        'WatchSessions':
        grpc.unary_stream_rpc_method_handler(
            servicer.WatchSessions,
            request_deserializer=grid_dot_v1_dot_session__service__pb2.WatchSessionsRequest.FromString,
            response_serializer=grid_dot_v1_dot_session__service__pb2.WatchSessionsResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler('grid.v1.SessionService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler, ))
